import json

from google.cloud.storage import Client
from pydantic import Field

from platzky.db.db import DBConfig
from platzky.db.json_db import Json


def db_config_type():
    return GoogleJsonDbConfig


class GoogleJsonDbConfig(DBConfig):
    bucket_name: str = Field(alias="BUCKET_NAME")
    source_blob_name: str = Field(alias="SOURCE_BLOB_NAME")


def db_from_config(config: GoogleJsonDbConfig):
    return GoogleJsonDb(config.bucket_name, config.source_blob_name)


def get_db(config):
    google_json_db_config = GoogleJsonDbConfig.model_validate(config)
    return GoogleJsonDb(google_json_db_config.bucket_name, google_json_db_config.source_blob_name)


def get_blob(bucket_name, source_blob_name):
    storage_client = Client()
    bucket = storage_client.bucket(bucket_name)
    return bucket.blob(source_blob_name)


def get_data(blob):
    raw_data = blob.download_as_text(client=None)
    return json.loads(raw_data)


class GoogleJsonDb(Json):
    def __init__(self, bucket_name, source_blob_name):
        self.bucket_name = bucket_name
        self.source_blob_name = source_blob_name

        self.blob = get_blob(self.bucket_name, self.source_blob_name)
        data = get_data(self.blob)
        super().__init__(data)

        self.module_name = "google_json_db"
        self.db_name = "GoogleJsonDb"



import json

from pydantic import Field

from platzky.db.db import DBConfig
from platzky.db.json_db import Json


def db_config_type():
    return JsonFileDbConfig


class JsonFileDbConfig(DBConfig):
    path: str = Field(alias="PATH")


def get_db(config):
    json_file_db_config = JsonFileDbConfig.model_validate(config)
    return JsonFile(json_file_db_config.path)


def db_from_config(config: JsonFileDbConfig):
    return JsonFile(config.path)


class JsonFile(Json):
    def __init__(self, path: str):
        self.data_file_path = path
        with open(self.data_file_path) as json_file:
            data = json.load(json_file)
            super().__init__(data)
        self.module_name = "json_file_db"
        self.db_name = "JsonFileDb"

    def __save_file(self):
        with open(self.data_file_path, "w") as json_file:
            json.dump(self.data, json_file)

    def add_comment(self, author_name, comment, post_slug):
        super().add_comment(author_name, comment, post_slug)
        self.__save_file()



import typing as t
import urllib.parse

from flask import redirect, render_template, request, session
from flask_minify import Minify

from platzky.admin import admin
from platzky.blog import blog
from platzky.config import (
    Config,
    languages_dict,
)
from platzky.db.db_loader import get_db
from platzky.engine import Engine
from platzky.plugin.plugin_loader import plugify
from platzky.seo import seo
from platzky.www_handler import redirect_nonwww_to_www, redirect_www_to_nonwww


def create_engine(config: Config, db) -> Engine:
    app = Engine(config, db, __name__)

    @app.before_request
    def handle_www_redirection():
        if config.use_www:
            return redirect_nonwww_to_www()
        else:
            return redirect_www_to_nonwww()

    def get_langs_domain(lang: str) -> t.Optional[str]:
        lang_cfg = config.languages.get(lang)
        if lang_cfg is None:
            return None
        return lang_cfg.domain

    @app.route("/lang/<string:lang>", methods=["GET"])
    def change_language(lang):
        if new_domain := get_langs_domain(lang):
            return redirect("http://" + new_domain, code=301)
        else:
            session["language"] = lang
            return redirect(request.referrer)

    def url_link(x: str) -> str:
        return urllib.parse.quote(x, safe="")

    @app.context_processor
    def utils():
        locale = app.get_locale()
        flag = lang.flag if (lang := config.languages.get(locale)) is not None else ""
        country = lang.country if (lang := config.languages.get(locale)) is not None else ""
        return {
            "app_name": config.app_name,
            "app_description": app.db.get_app_description(locale) or config.app_name,
            "languages": languages_dict(config.languages),
            "current_flag": flag,
            "current_lang_country": country,
            "current_language": locale,
            "url_link": url_link,
            "menu_items": app.db.get_menu_items_in_lang(locale),
            "logo_url": app.db.get_logo_url(),
            "favicon_url": app.db.get_favicon_url(),
            "font": app.db.get_font(),
            "primary_color": app.db.get_primary_color(),
            "secondary_color": app.db.get_secondary_color(),
        }

    @app.context_processor
    def dynamic_body():
        return {"dynamic_body": app.dynamic_body}

    @app.context_processor
    def dynamic_head():
        return {"dynamic_head": app.dynamic_head}

    @app.errorhandler(404)
    def page_not_found(e):
        return render_template("404.html", title="404"), 404

    return plugify(app)


def create_app_from_config(config: Config) -> Engine:
    db = get_db(config.db)
    engine = create_engine(config, db)

    admin_blueprint = admin.create_admin_blueprint(
        login_methods=engine.login_methods, db=engine.db, locale_func=engine.get_locale
    )

    if config.feature_flags and config.feature_flags.get("FAKE_LOGIN", False):
        from platzky.admin.fake_login import get_fake_login_html, setup_fake_login_routes

        engine.login_methods.append(get_fake_login_html())
        admin_blueprint = setup_fake_login_routes(admin_blueprint)

    blog_blueprint = blog.create_blog_blueprint(
        db=engine.db,
        blog_prefix=config.blog_prefix,
        locale_func=engine.get_locale,
    )
    seo_blueprint = seo.create_seo_blueprint(
        db=engine.db, config=engine.config, locale_func=engine.get_locale
    )
    engine.register_blueprint(admin_blueprint)
    engine.register_blueprint(blog_blueprint)
    engine.register_blueprint(seo_blueprint)

    Minify(app=engine, html=True, js=True, cssless=True)
    return engine


def create_app(config_path: str) -> Engine:
    config = Config.parse_yaml(config_path)
    return create_app_from_config(config)



import datetime

import humanize
from pydantic import BaseModel


class Image(BaseModel):
    url: str = ""
    alternateText: str = ""


class MenuItem(BaseModel):
    name: str
    url: str


class Comment(BaseModel):
    author: str
    comment: str
    date: str  # TODO change its type to datetime

    @property
    def time_delta(self) -> str:
        now = datetime.datetime.now()
        date = datetime.datetime.strptime(self.date.split(".")[0], "%Y-%m-%dT%H:%M:%S")
        return humanize.naturaltime(now - date)


class Post(BaseModel):
    author: str
    slug: str
    title: str
    contentInMarkdown: str
    comments: list[Comment]
    excerpt: str
    tags: list[str]
    language: str
    coverImage: Image
    date: str

    def __lt__(self, other):
        if isinstance(other, Post):
            return self.date < other.date
        raise NotImplementedError("Posts can only be compared with other posts")


Page = Post


class Color(BaseModel):
    def __init__(self, r: int = 0, g: int = 0, b: int = 0, a: int = 255):
        if not (0 <= r <= 255):
            raise ValueError("r must be between 0 and 255")
        if not (0 <= g <= 255):
            raise ValueError("g must be between 0 and 255")
        if not (0 <= b <= 255):
            raise ValueError("b must be between 0 and 255")
        if not (0 <= a <= 255):
            raise ValueError("a must be between 0 and 255")
        super().__init__(r=r, g=g, b=b, a=a)

    r: int
    g: int
    b: int
    a: int



# TODO rename file, extract it to another library, remove qgl and aiohttp from dependencies


from gql import Client, gql
from gql.transport.aiohttp import AIOHTTPTransport
from gql.transport.exceptions import TransportQueryError
from pydantic import Field

from platzky.db.db import DB, DBConfig
from platzky.models import Color, Post


def db_config_type():
    return GraphQlDbConfig


class GraphQlDbConfig(DBConfig):
    endpoint: str = Field(alias="CMS_ENDPOINT")
    token: str = Field(alias="CMS_TOKEN")


def get_db(config: GraphQlDbConfig):
    return GraphQL(config.endpoint, config.token)


def db_from_config(config: GraphQlDbConfig):
    return GraphQL(config.endpoint, config.token)


def _standarize_comment(
    comment,
):
    return {
        "author": comment["author"],
        "comment": comment["comment"],
        "date": comment["createdAt"],
    }


def _standarize_post(post):
    return {
        "author": post["author"]["name"],
        "slug": post["slug"],
        "title": post["title"],
        "excerpt": post["excerpt"],
        "contentInMarkdown": post["contentInRichText"]["html"],
        "comments": [_standarize_comment(comment) for comment in post["comments"]],
        "tags": post["tags"],
        "language": post["language"],
        "coverImage": {
            "url": post["coverImage"]["image"]["url"],
        },
        "date": post["date"],
    }


class GraphQL(DB):
    def __init__(self, endpoint, token):
        self.module_name = "graph_ql_db"
        self.db_name = "GraphQLDb"
        full_token = "bearer " + token
        transport = AIOHTTPTransport(url=endpoint, headers={"Authorization": full_token})
        self.client = Client(transport=transport)
        super().__init__()

    def get_all_posts(self, lang):
        all_posts = gql(
            """
            query MyQuery($lang: Lang!) {
              posts(where: {language: $lang},  orderBy: date_DESC, stage: PUBLISHED){
                createdAt
                author {
                    name
                }
                contentInRichText {
                    html
                    }
                comments {
                  comment
                  author
                  createdAt
                  }
                date
                title
                excerpt
                slug
                tags
                language
                coverImage {
                  alternateText
                  image {
                    url
                  }
                }
              }
            }
            """
        )
        raw_ql_posts = self.client.execute(all_posts, variable_values={"lang": lang})["posts"]

        return [Post.model_validate(_standarize_post(post)) for post in raw_ql_posts]

    def get_menu_items_in_lang(self, lang):
        menu_items = []
        try:
            menu_items_with_lang = gql(
                """
                query MyQuery($lang: Lang!) {
                  menuItems(where: {language: $lang}, stage: PUBLISHED){
                    name
                    url
                  }
                }
                """
            )
            menu_items = self.client.execute(
                menu_items_with_lang, variable_values={"language": lang}
            )

        # TODO remove try except block after bumping up version
        # now it's backwards compatible with older versions
        except TransportQueryError:
            menu_items_without_lang = gql(
                """
                query MyQuery {
                  menuItems(stage: PUBLISHED){
                    name
                    url
                  }
                }
                """
            )
            menu_items = self.client.execute(menu_items_without_lang)

        return menu_items["menuItems"]

    def get_post(self, slug):
        post = gql(
            """
            query MyQuery($slug: String!) {
              post(where: {slug: $slug}, stage: PUBLISHED) {
                date
                language
                title
                slug
                author {
                    name
                }
                contentInRichText {
                  markdown
                  html
                }
                excerpt
                tags
                coverImage {
                  alternateText
                  image {
                    url
                  }
                }
                comments {
                    author
                    comment
                    date: createdAt
                }
              }
            }
            """
        )

        post_raw = self.client.execute(post, variable_values={"slug": slug})["post"]
        return Post.model_validate(_standarize_post(post_raw))

    # TODO Cleanup page logic of internationalization (now it depends on translation of slugs)
    def get_page(self, slug):
        post = gql(
            """
            query MyQuery ($slug: String!){
              page(where: {slug: $slug}, stage: PUBLISHED) {
                title
                contentInMarkdown
                coverImage
                {
                    url
                }
              }
            }
            """
        )
        return self.client.execute(post, variable_values={"slug": slug})["page"]

    def get_posts_by_tag(self, tag, lang):
        post = gql(
            """
            query MyQuery ($tag: String!, $lang: Lang!){
              posts(where: {tags_contains_some: [$tag], language: $lang}, stage: PUBLISHED) {
                    tags
                    title
                    slug
                    excerpt
                    date
                    coverImage {
                      alternateText
                      image {
                        url
                      }
                    }
              }
            }
            """
        )
        return self.client.execute(post, variable_values={"tag": tag, "lang": lang})["posts"]

    def add_comment(self, author_name, comment, post_slug):
        add_comment = gql(
            """
            mutation MyMutation($author: String!, $comment: String!, $slug: String!) {
                createComment(
                    data: {
                        author: $author,
                        comment: $comment,
                        post: {connect: {slug: $slug}}
                    }
                ) {
                    id
                }
            }
            """
        )
        self.client.execute(
            add_comment,
            variable_values={
                "author": author_name,
                "comment": comment,
                "slug": post_slug,
            },
        )

    def get_font(self):
        return str("")

    def get_logo_url(self):
        logo = gql(
            """
            query myquery {
              logos(stage: PUBLISHED) {
              logo {
                  alternateText
                  image {
                    url
                  }
                }
              }
            }
            """
        )
        try:
            return self.client.execute(logo)["logos"][0]["logo"]["image"]["url"]
        except IndexError:
            return ""

    def get_app_description(self, lang):
        description_query = gql(
            """
            query myquery($lang: Lang!) {
              applicationSetups(where: {language: $lang}, stage: PUBLISHED) {
                applicationDescription
              }
            }
            """
        )

        return self.client.execute(description_query, variable_values={"lang": lang})[
            "applicationSetups"
        ][0].get("applicationDescription", None)

    def get_favicon_url(self):
        favicon = gql(
            """
            query myquery {
              favicons(stage: PUBLISHED) {
              favicon {
                url
                }
              }
            }
            """
        )

        return self.client.execute(favicon)["favicons"][0]["favicon"]["url"]

    def get_primary_color(self) -> Color:
        return Color()

    def get_secondary_color(self):
        return Color()

    def get_plugins_data(self):
        plugins_data = gql(
            """
            query MyQuery {
              pluginConfigs(stage: PUBLISHED) {
                name
                config
              }
            }
            """
        )
        return self.client.execute(plugins_data)["pluginConfigs"]



import datetime
from typing import Any, Dict

from pydantic import Field

from platzky.db.db import DB, DBConfig
from platzky.models import MenuItem, Post


def db_config_type():
    return JsonDbConfig


class JsonDbConfig(DBConfig):
    data: Dict[str, Any] = Field(alias="DATA")


def get_db(config):
    json_db_config = JsonDbConfig.model_validate(config)
    return Json(json_db_config.data)


def db_from_config(config: JsonDbConfig):
    return Json(config.data)


# TODO make all language specific methods to be available without language
# this will allow to have a default language and if there is one language
# there will be no need to pass it to the method or in db


class Json(DB):
    def __init__(self, data: Dict[str, Any]):
        super().__init__()
        self.data: Dict[str, Any] = data
        self.module_name = "json_db"
        self.db_name = "JsonDb"

    def get_app_description(self, lang):
        description = self._get_site_content().get("app_description", {})
        return description.get(lang, None)

    def get_all_posts(self, lang):
        return [
            Post.model_validate(post)
            for post in self._get_site_content().get("posts", ())
            if post["language"] == lang
        ]

    def get_post(self, slug: str) -> Post:
        """Returns a post matching the given slug."""
        all_posts = self._get_site_content().get("posts")
        if all_posts is None:
            raise ValueError("Posts data is missing")
        wanted_post = next((post for post in all_posts if post["slug"] == slug), None)
        if wanted_post is None:
            raise ValueError(f"Post with slug {slug} not found")
        return Post.model_validate(wanted_post)

    # TODO: add test for non-existing page
    def get_page(self, slug):
        list_of_pages = (
            page for page in self._get_site_content().get("pages") if page["slug"] == slug
        )
        page = Post.model_validate(next(list_of_pages))
        return page

    def get_menu_items_in_lang(self, lang) -> list[MenuItem]:
        menu_items_raw = self._get_site_content().get("menu_items", {})
        items_in_lang = menu_items_raw.get(lang, {})

        menu_items_list = [MenuItem.model_validate(x) for x in items_in_lang]
        return menu_items_list

    def get_posts_by_tag(self, tag, lang):
        return (
            post
            for post in self._get_site_content()["posts"]
            if tag in post["tags"] and post["language"] == lang
        )

    def _get_site_content(self):
        content = self.data.get("site_content")
        if content is None:
            raise Exception("Content should not be None")
        return content

    def get_logo_url(self):
        return self._get_site_content().get("logo_url", "")

    def get_favicon_url(self):
        return self._get_site_content().get("favicon_url", "")

    def get_font(self) -> str:
        return self._get_site_content().get("font", "")

    def get_primary_color(self):
        return self._get_site_content().get("primary_color", "white")

    def get_secondary_color(self):
        return self._get_site_content().get("secondary_color", "navy")

    def add_comment(self, author_name, comment, post_slug):
        comment = {
            "author": str(author_name),
            "comment": str(comment),
            "date": datetime.datetime.now().strftime("%Y-%m-%dT%H:%M:%S"),
        }

        post_index = next(
            i
            for i in range(len(self._get_site_content()["posts"]))
            if self._get_site_content()["posts"][i]["slug"] == post_slug
        )
        self._get_site_content()["posts"][post_index]["comments"].append(comment)

    def get_plugins_data(self):
        return self.data.get("plugins", [])



import json

import requests
from github import Github
from pydantic import Field

from platzky.db.db import DBConfig
from platzky.db.json_db import Json as JsonDB


def db_config_type():
    return GithubJsonDbConfig


class GithubJsonDbConfig(DBConfig):
    github_token: str = Field(alias="GITHUB_TOKEN")
    repo_name: str = Field(alias="REPO_NAME")
    path_to_file: str = Field(alias="PATH_TO_FILE")
    branch_name: str = Field(alias="BRANCH_NAME", default="main")


def db_from_config(config: GithubJsonDbConfig):
    return GithubJsonDb(
        config.github_token, config.repo_name, config.branch_name, config.path_to_file
    )


def get_db(config):
    github_json_db_config = GithubJsonDbConfig.model_validate(config)
    return GithubJsonDb(
        github_json_db_config.github_token,
        github_json_db_config.repo_name,
        github_json_db_config.branch_name,
        github_json_db_config.path_to_file,
    )


class GithubJsonDb(JsonDB):
    def __init__(self, github_token: str, repo_name: str, branch_name: str, path_to_file: str):
        self.branch_name = branch_name
        self.repo = Github(github_token).get_repo(repo_name)
        self.file_path = path_to_file

        try:
            file_content = self.repo.get_contents(self.file_path, ref=self.branch_name)

            if isinstance(file_content, list):
                raise ValueError(f"Path '{self.file_path}' points to a directory, not a file")

            if file_content.content:
                raw_data = file_content.decoded_content.decode("utf-8")
            else:

                download_url = file_content.download_url
                response = requests.get(download_url, timeout=40)
                response.raise_for_status()
                raw_data = response.text

            self.data = json.loads(raw_data)

        except (json.JSONDecodeError, requests.RequestException) as e:
            raise ValueError(f"Error parsing JSON content: {e}")
        except Exception as e:
            raise ValueError(f"Error retrieving GitHub content: {e}")

        super().__init__(self.data)

        self.module_name = "github_json_db"
        self.db_name = "GithubJsonDb"



import logging
from abc import ABC, abstractmethod
from typing import Any, Dict, Generic, Type, TypeVar

from pydantic import BaseModel, ConfigDict

from platzky.platzky import Engine as PlatzkyEngine

logger = logging.getLogger(__name__)


class PluginError(Exception):
    """Exception raised for plugin-related errors."""

    pass


class ConfigPluginError(PluginError):
    """Exception raised for plugin configuration-related errors."""

    pass


class PluginBaseConfig(BaseModel):
    """Base Pydantic model for plugin configurations.

    Plugin developers should extend this class to define their own configuration schema.
    """

    model_config = ConfigDict(extra="allow")


T = TypeVar("T", bound=PluginBaseConfig)


class PluginBase(Generic[T], ABC):
    """Abstract base class for plugins.

    Plugin developers must extend this class to implement their plugins.
    """

    @classmethod
    def get_config_model(cls) -> Type[PluginBaseConfig]:
        return PluginBaseConfig

    def __init__(self, config: Dict[str, Any]):
        try:
            config_class = self.get_config_model()
            self.config = config_class.model_validate(config)
        except Exception as e:
            raise ConfigPluginError(f"Invalid configuration: {e}") from e

    @abstractmethod
    def process(self, app: PlatzkyEngine) -> PlatzkyEngine:
        """Process the plugin with the given app.

        Args:
            app: The Flask application instance

        Returns:
            Platzky Engine with processed plugins

        Raises:
            PluginError: If plugin processing fails
        """
        pass



from abc import ABC, abstractmethod
from functools import partial
from typing import Any, Callable

from pydantic import BaseModel, Field

from platzky.models import Color, MenuItem, Page, Post


class DB(ABC):
    db_name: str = "DB"
    module_name: str = "db"
    config_type: type

    def __init_subclass__(cls, *args, **kw):
        """Check that all methods defined in the subclass exist in the superclasses.
        This is to prevent subclasses from splitting up DB interface.
        """
        super().__init_subclass__(*args, **kw)
        for name in cls.__dict__:
            if name.startswith("_"):
                continue
            for superclass in cls.__mro__[1:]:
                if name in dir(superclass):
                    break
            else:
                raise TypeError(
                    f"Method {name} defined in {cls.__name__} does not exist in superclasses"
                )

    def extend(self, function_name: str, function: Callable[..., Any]) -> None:
        """
        Add a function to the DB object. The function must take the DB object as first parameter.

        Parameters:
        function_name (str): The name of the function to add.
        function (Callable): The function to add to the DB object.
        """
        if not callable(function):
            raise ValueError(f"The provided func for '{function_name}' is not callable.")
        try:
            bound_function = partial(function, self)
            setattr(self, function_name, bound_function)
        except Exception as e:
            raise ValueError(f"Failed to extend DB with function {function_name}: {e}")

    @abstractmethod
    def get_app_description(self, lang) -> str:
        pass

    @abstractmethod
    def get_all_posts(self, lang) -> list[Post]:
        pass

    @abstractmethod
    def get_menu_items_in_lang(self, lang) -> list[MenuItem]:
        pass

    @abstractmethod
    def get_post(self, slug) -> Post:
        pass

    @abstractmethod
    def get_page(self, slug) -> Page:
        pass

    @abstractmethod
    def get_posts_by_tag(self, tag, lang) -> Any:
        pass

    @abstractmethod
    def add_comment(self, author_name, comment, post_slug) -> None:
        pass

    @abstractmethod
    def get_logo_url(self) -> str:  # TODO provide alternative text along with the URL of logo
        pass

    @abstractmethod
    def get_favicon_url(self) -> str:
        pass

    @abstractmethod
    def get_primary_color(self) -> Color:
        pass

    @abstractmethod
    def get_secondary_color(self) -> Color:
        pass

    @abstractmethod
    def get_plugins_data(self) -> list[Any]:
        pass

    @abstractmethod
    def get_font(self) -> str:
        pass


class DBConfig(BaseModel):
    type: str = Field(alias="TYPE")



import typing as t
import urllib.parse
from os.path import dirname

from flask import Blueprint, current_app, make_response, render_template, request


def create_seo_blueprint(db, config: dict[str, t.Any], locale_func: t.Callable[[], str]):
    seo = Blueprint(
        "seo",
        __name__,
        url_prefix=config["SEO_PREFIX"],
        template_folder=f"{dirname(__file__)}/../templates",
    )

    @seo.route("/robots.txt")
    def robots():
        robots_response = render_template("robots.txt", domain=request.host, mimetype="text/plain")
        response = make_response(robots_response)
        response.headers["Content-Type"] = "text/plain"
        return response

    def get_blog_entries(host_base, lang, db, blog_prefix):
        dynamic_urls = list()
        print(blog_prefix)
        for post in db.get_all_posts(
            lang
        ):  # TODO add get_list_of_posts for faster getting just list of it
            slug = post.slug
            datet = post.date.split("T")[0]
            url = {"loc": f"{host_base}{blog_prefix}/{slug}", "lastmod": datet}
            dynamic_urls.append(url)
        return dynamic_urls

    @seo.route("/sitemap.xml")  # TODO try to replace sitemap logic with flask-sitemap module
    def sitemap():
        """
        Route to dynamically generate a sitemap of your website/application.
        lastmod and priority tags omitted on static pages.
        lastmod included on dynamic content such as seo posts.
        """
        lang = locale_func()

        global url
        host_components = urllib.parse.urlparse(request.host_url)
        host_base = host_components.scheme + "://" + host_components.netloc

        # Static routes with static content
        static_urls = list()
        for rule in current_app.url_map.iter_rules():
            if rule.methods is not None and "GET" in rule.methods and len(rule.arguments) == 0:
                url = {"loc": f"{host_base}{rule!s}"}
                static_urls.append(url)

        dynamic_urls = get_blog_entries(host_base, lang, db, config["BLOG_PREFIX"])

        statics = list({v["loc"]: v for v in static_urls}.values())
        dynamics = list({v["loc"]: v for v in dynamic_urls}.values())
        xml_sitemap = render_template(
            "sitemap.xml",
            static_urls=statics,
            dynamic_urls=dynamics,
            host_base=host_base,
        )
        response = make_response(xml_sitemap)
        response.headers["Content-Type"] = "application/xml"
        return response

    return seo


# TODO add tests which would check that sitemap is different for different languages



import urllib.parse

from flask import redirect, request


def redirect_nonwww_to_www():
    """Redirect non-www requests to www."""
    urlparts = urllib.parse.urlparse(request.url)
    if not urlparts.netloc.startswith("www."):
        urlparts = urlparts._replace(netloc=f"www.{urlparts.netloc}")
        url = urllib.parse.urlunparse(urlparts)
        return redirect(url, code=301)


def redirect_www_to_nonwww():
    """Redirect www requests to non-www."""
    urlparts = urllib.parse.urlparse(request.url)
    if urlparts.netloc.startswith("www."):
        urlparts = urlparts._replace(netloc=urlparts.netloc.removeprefix("www."))
        url = urllib.parse.urlunparse(urlparts)
        return redirect(url, code=302)



"""
Fake login functionality for development environments only.

WARNING: This module provides fake login functionality and should NEVER be used in production
environments as it bypasses proper authentication and authorization controls.
"""

import os
from typing import Any, Callable

from flask import Blueprint, flash, redirect, session, url_for
from markupsafe import Markup

ROLE_ADMIN = "admin"
ROLE_NONADMIN = "nonadmin"
VALID_ROLES = [ROLE_ADMIN, ROLE_NONADMIN]


def get_fake_login_html() -> Callable[[], str]:
    """Return a callable that generates HTML for fake login buttons."""

    def generate_html() -> str:
        admin_url = url_for("admin.handle_fake_login", role="admin")
        nonadmin_url = url_for("admin.handle_fake_login", role="nonadmin")

        # Rest of the code remains the same
        html = f"""
        <div class="col-md-6 mb-4">
          <div class="card">
            <div class="card-header">
              Development Login
            </div>
            <div class="card-body">
              <p class="text-danger"><strong>Warning:</strong> For development only</p>
              <div class="d-flex justify-content-around">
                <form method="post" action="{admin_url}" style="display: inline;">
                  <input type="hidden" name="csrf_token" value="{{ csrf_token() }}">
                  <button type="submit" class="btn btn-primary">Login as Admin</button>
                </form>
                <form method="post" action="{nonadmin_url}" style="display: inline;">
                  <input type="hidden" name="csrf_token" value="{{ csrf_token() }}">
                  <button type="submit" class="btn btn-secondary">Login as Non-Admin</button>
                </form>
              </div>
            </div>
          </div>
        </div>
        """
        return Markup(html)

    return generate_html


def setup_fake_login_routes(admin_blueprint: Blueprint) -> Blueprint:
    """Add fake login routes to the provided admin_blueprint."""

    env = os.environ
    is_testing = "PYTEST_CURRENT_TEST" in env.keys() or env.get("FLASK_DEBUG") in (
        "1",
        "true",
        "True",
        True,
    )

    if not is_testing:
        raise RuntimeError(
            "SECURITY ERROR: Fake login routes are enabled outside of a testing environment! "
            "This functionality must only be used during development or testing."
        )

    @admin_blueprint.route("/fake-login/<role>", methods=["POST"])
    def handle_fake_login(role: str) -> Any:
        if role not in VALID_ROLES:
            flash(f"Invalid role: {role}. Must be one of: {', '.join(VALID_ROLES)}", "error")
            return redirect(url_for("admin.admin_panel_home"))
        if role == ROLE_ADMIN:
            session["user"] = {"username": ROLE_ADMIN, "role": ROLE_ADMIN}
        else:
            session["user"] = {"username": "user", "role": ROLE_NONADMIN}
        return redirect(url_for("admin.admin_panel_home"))

    return admin_blueprint



from os.path import dirname

from flask import Blueprint, make_response, render_template, request
from markupsafe import Markup

from . import comment_form


def create_blog_blueprint(db, blog_prefix: str, locale_func):
    url_prefix = blog_prefix
    blog = Blueprint(
        "blog",
        __name__,
        url_prefix=url_prefix,
        template_folder=f"{dirname(__file__)}/../templates",
    )

    @blog.app_template_filter()
    def markdown(text):
        return Markup(text)

    @blog.errorhandler(404)
    def page_not_found(e):
        return render_template("404.html", title="404"), 404

    @blog.route("/", methods=["GET"])
    def all_posts():
        lang = locale_func()
        posts = db.get_all_posts(lang)
        if not posts:
            return page_not_found("no posts")
        posts_sorted = sorted(posts, reverse=True)
        return render_template("blog.html", posts=posts_sorted)

    @blog.route("/feed", methods=["GET"])
    def get_feed():
        lang = locale_func()
        response = make_response(render_template("feed.xml", posts=db.get_all_posts(lang)))
        response.headers["Content-Type"] = "application/xml"
        return response

    @blog.route("/<post_slug>", methods=["POST"])
    def post_comment(post_slug):
        comment = request.form.to_dict()
        db.add_comment(
            post_slug=post_slug,
            author_name=comment["author_name"],
            comment=comment["comment"],
        )
        return get_post(post_slug=post_slug)

    @blog.route("/<post_slug>", methods=["GET"])
    def get_post(post_slug):
        try:
            post = db.get_post(post_slug)
            return render_template(
                "post.html",
                post=post,
                post_slug=post_slug,
                form=comment_form.CommentForm(),
                comment_sent=request.args.get("comment_sent"),
            )
        except ValueError:
            return page_not_found(f"no post with slug {post_slug}")
        except Exception as e:
            return page_not_found(str(e))

    @blog.route("/page/<path:page_slug>", methods=["GET"])
    def get_page(
        page_slug,
    ):  # TODO refactor to share code with get_post since they are very similar
        try:
            page = db.get_page(page_slug)
            if cover_image := page.coverImage:
                cover_image_url = cover_image.url
            else:
                cover_image_url = None
            return render_template("page.html", page=page, cover_image=cover_image_url)
        except ValueError:
            return page_not_found("no page with slug {page_slug}")
        except Exception as e:
            return page_not_found(str(e))

    @blog.route("/tag/<path:tag>", methods=["GET"])
    def get_posts_from_tag(tag):
        lang = locale_func()
        posts = db.get_posts_by_tag(tag, lang)
        return render_template("blog.html", posts=posts, subtitle=f" - tag: {tag}")

    return blog



import os

from flask import Flask, request, session
from flask_babel import Babel

from platzky.config import Config


class Engine(Flask):
    def __init__(self, config: Config, db, import_name):
        super().__init__(import_name)
        self.config.from_mapping(config.model_dump(by_alias=True))
        self.db = db
        self.notifiers = []
        self.login_methods = []
        self.dynamic_body = ""
        self.dynamic_head = ""
        directory = os.path.dirname(os.path.realpath(__file__))
        locale_dir = os.path.join(directory, "locale")
        config.translation_directories.append(locale_dir)
        babel_translation_directories = ";".join(config.translation_directories)
        self.babel = Babel(
            self,
            locale_selector=self.get_locale,
            default_translation_directories=babel_translation_directories,
        )

    def notify(self, message: str):
        for notifier in self.notifiers:
            notifier(message)

    def add_notifier(self, notifier):
        self.notifiers.append(notifier)

    # TODO login_method should be interface
    def add_login_method(self, login_method):
        self.login_methods.append(login_method)

    def add_dynamic_body(self, body: str):
        self.dynamic_body += body

    def add_dynamic_head(self, body: str):
        self.dynamic_head += body

    def get_locale(self) -> str:
        domain = request.headers.get("Host", "localhost")
        domain_to_lang = self.config.get("DOMAIN_TO_LANG")

        languages = self.config.get("LANGUAGES", {}).keys()
        backup_lang = session.get(
            "language",
            request.accept_languages.best_match(languages, "en"),
        )

        if domain_to_lang:
            lang = domain_to_lang.get(domain, backup_lang)
        else:
            lang = backup_lang

        session["language"] = lang
        return lang



from os.path import dirname

from flask import Blueprint, render_template, session


def create_admin_blueprint(login_methods, db, locale_func):
    admin = Blueprint(
        "admin",
        __name__,
        url_prefix="/admin",
        template_folder=f"{dirname(__file__)}/templates",
    )

    @admin.route("/", methods=["GET"])
    def admin_panel_home():
        user = session.get("user", None)

        if not user:
            return render_template("login.html", login_methods=login_methods)

        cms_modules = {"plugins": [plugin.get("name") for plugin in db.get_plugins_data()]}
        return render_template("admin.html", user=user, cms_modules=cms_modules)

    @admin.route("/module/<module_name>", methods=["GET"])
    def module_settings(module_name):
        user = session.get("user", None)

        if not user:
            return render_template("login.html", login_methods=login_methods)

        return render_template("module.html", user=user, module_name=module_name)

    return admin



import importlib.util
import os
import sys
from os.path import abspath, dirname


def get_db(db_config):
    db_name = db_config.type
    db = get_db_module(db_name)
    return db.db_from_config(db_config)


def get_db_module(db_type):
    """
    Load db module from db_type
    This function is used to load db module dynamically as it is specified in config file.
    :param db_type: name of db module
    :return: db module
    """
    db_dir = dirname(abspath(__file__))
    parent_module_name = ".".join(__name__.split(".")[:-1])
    module_name = f"{parent_module_name}.{db_type}_db"
    spec = importlib.util.spec_from_file_location(
        module_name, os.path.join(db_dir, f"{db_type}_db.py")
    )
    assert spec is not None
    db = importlib.util.module_from_spec(spec)
    sys.modules[f"{db_type}_db"] = db
    assert spec.loader is not None
    spec.loader.exec_module(db)

    return db



import sys
import typing as t

import yaml
from pydantic import BaseModel, ConfigDict, Field

from .db.db import DBConfig
from .db.db_loader import get_db_module


class StrictBaseModel(BaseModel):
    model_config = ConfigDict(frozen=True)


class LanguageConfig(StrictBaseModel):
    name: str = Field(alias="name")
    flag: str = Field(alias="flag")
    country: str = Field(alias="country")
    domain: t.Optional[str] = Field(default=None, alias="domain")


Languages = dict[str, LanguageConfig]
LanguagesMapping = t.Mapping[str, t.Mapping[str, str]]


def languages_dict(languages: Languages) -> LanguagesMapping:
    return {name: lang.model_dump() for name, lang in languages.items()}


class Config(StrictBaseModel):
    app_name: str = Field(alias="APP_NAME")
    secret_key: str = Field(alias="SECRET_KEY")
    db: DBConfig = Field(alias="DB")
    use_www: bool = Field(default=True, alias="USE_WWW")
    seo_prefix: str = Field(default="/", alias="SEO_PREFIX")
    blog_prefix: str = Field(default="/", alias="BLOG_PREFIX")
    languages: Languages = Field(default_factory=dict, alias="LANGUAGES")
    domain_to_lang: dict[str, str] = Field(default_factory=dict, alias="DOMAIN_TO_LANG")
    translation_directories: list[str] = Field(
        default_factory=list,
        alias="TRANSLATION_DIRECTORIES",
    )
    debug: bool = Field(default=False, alias="DEBUG")
    testing: bool = Field(default=False, alias="TESTING")
    feature_flags: t.Optional[dict[str, bool]] = Field(default_factory=dict, alias="FEATURE_FLAGS")

    @classmethod
    def model_validate(
        cls,
        obj: t.Any,
        *,
        strict: bool | None = None,
        from_attributes: bool | None = None,
        context: dict[str, t.Any] | None = None,
    ) -> "Config":
        db_cfg_type = get_db_module(obj["DB"]["TYPE"]).db_config_type()
        obj["DB"] = db_cfg_type.model_validate(obj["DB"])
        return super().model_validate(
            obj, strict=strict, from_attributes=from_attributes, context=context
        )

    @classmethod
    def parse_yaml(cls, path: str) -> "Config":
        try:
            with open(path, "r") as f:
                return cls.model_validate(yaml.safe_load(f))
        except FileNotFoundError:
            print(f"Config file not found: {path}", file=sys.stderr)
            raise SystemExit(1)



import importlib.util
import inspect
import logging
from typing import Any, Optional, Type

import deprecation

from platzky.engine import Engine
from platzky.plugin.plugin import PluginBase, PluginError

logger = logging.getLogger(__name__)


def find_plugin(plugin_name: str) -> Any:
    """Find plugin by name and return it as module.

    Args:
        plugin_name: name of plugin to find

    Raises:
        PluginError: if plugin cannot be imported

    Returns:
        module of plugin
    """
    try:
        return importlib.import_module(f"platzky_{plugin_name}")
    except ImportError as e:
        raise PluginError(
            f"Plugin {plugin_name} not found. Ensure it's installed and follows "
            f"the 'platzky_<plugin_name>' naming convention"
        ) from e


def _is_class_plugin(plugin_module: Any) -> Optional[Type[PluginBase[Any]]]:
    """Check if the plugin module contains a PluginBase implementation.

    Args:
        plugin_module: The imported plugin module

    Returns:
        The plugin class if found, None otherwise
    """
    # Look for classes in the module that inherit from PluginBase
    for _, obj in inspect.getmembers(plugin_module):
        if inspect.isclass(obj) and issubclass(obj, PluginBase) and obj != PluginBase:
            return obj
    return None


@deprecation.deprecated(
    deprecated_in="0.3.1",
    removed_in="0.4.0",
    current_version=None,  # You should replace this with the current version
    details="Legacy plugin style using the entrypoint process() function is deprecated. "
    "Please migrate to the PluginBase interface.",
)
def _process_legacy_plugin(plugin_module, app, plugin_config, plugin_name):
    """Process a legacy plugin using the entrypoint approach."""
    app = plugin_module.process(app, plugin_config)
    logger.info(f"Processed legacy plugin: {plugin_name}")
    return app


def plugify(app: Engine) -> Engine:
    """Load plugins and run their entrypoints.

    Supports both class-based plugins (PluginBase) and legacy entrypoint plugins.

    Args:
        app: Platzky Engine instance

    Returns:
        Platzky Engine with processed plugins

    Raises:
        PluginError: if plugin processing fails
    """
    plugins_data = app.db.get_plugins_data()

    for plugin_data in plugins_data:
        plugin_config = plugin_data["config"]
        plugin_name = plugin_data["name"]

        try:
            plugin_module = find_plugin(plugin_name)

            # Check if this is a class-based plugin
            plugin_class = _is_class_plugin(plugin_module)

            if plugin_class:
                # Handle new class-based plugins
                plugin_instance = plugin_class(plugin_config)
                app = plugin_instance.process(app)
                logger.info(f"Processed class-based plugin: {plugin_name}")
            elif hasattr(plugin_module, "process"):
                # Handle legacy entrypoint plugins with deprecation warning
                app = _process_legacy_plugin(plugin_module, app, plugin_config, plugin_name)
            else:
                raise PluginError(
                    f"Plugin {plugin_name} doesn't implement either the PluginBase interface "
                    f"or provide a process() function"
                )

        except Exception as e:
            logger.error(f"Error processing plugin {plugin_name}: {e}")
            raise PluginError(f"Error processing plugin {plugin_name}: {e}") from e

    return app
